#include "extern.h"
#include "main.h"
#include "stm32f4xx_hal.h"
#include <stdbool.h>

#define CELSIUS20 2000 //2000 div 100 = 20°C
#define CELSIUS15 1500


#define START_PRESSURE_CONV 0  
#define READ_PRESSURE 1
#define START_TEMPERATURE_CONV 2
#define READ_TEMPERATURE 3
#define CALC_TEMPERATURE 4
#define BRANCH_LOW_HIGH_TEMPERATURE 5
#define HIGH_TEMPERATURE 50
#define LOW_TEMPERATURE 51
#define VERY_LOW_TEMPERATURE 52
#define DUMMY_HIGH_TEMPERATURE 56
#define DUMMY_LOW_TEMPERATURE 57 
#define SECOND_ORDER_COMP 6
#define OUTPUT_READY 7



//extern TIM_HandleTypeDef htim3;
extern I2C_HandleTypeDef hi2c1;
extern bool new_pressure_value;


// BUFFERS
	uint8_t tx_buffer[1];
	uint8_t rx_buffer[3] = {0};  //2 bytes for constant, third one for adc readings
	
/*
// CONSTANTS
	static uint16_t C1; // Pressure sensitivity | SENS T1 
	static uint16_t C2; // Pressure offset | OFF 
	static uint16_t C3; // Temperature coefficient of pressure sensitivity | TCS 
	static uint16_t C4; // Temperature coefficient of pressure offset | TCO 
	static uint16_t C5; // Reference temperature | T REF 
	static uint16_t C6; // Temperature coefficient of the temperature | TEMPSENS 
*/
	
// READINGS
	static uint32_t D1; // Pressure value
	static uint32_t D2; // Temperature value
	
// INTERMIDIATE VAR FOR TEMP COMPENSATION 
	static int32_t dT;
	static int32_t TEMP;
	
	
	//static int64_t Ti;
	static int64_t tmp_TEMPSQR; // così vengono fatti i conti una volta sola
	static int64_t OFF;
	static int64_t SENS;
	static int64_t OFFinterm;
	static int64_t SENSinterm;
	
	static uint32_t PRESSURE;
	//bool pressure_ready = 0;
	
	
	uint8_t STATE = 0;	
	


int pres_sensor(uint16_t C1,  uint16_t C2, uint16_t C3, uint16_t C4, uint16_t C5, uint16_t C6)
{	

	switch(STATE){
		
		case START_PRESSURE_CONV: //Pressure READING
			

			//init conversion
			tx_buffer[0] = 0x4A; // Convert D1 (OSR=8192) 18.08ms
			HAL_I2C_Master_Transmit(&hi2c1, 0xEC, tx_buffer, 1, 1000);		
			
			STATE = READ_PRESSURE;
			
		break;
		
		case READ_PRESSURE:
		
			// CMD READ ADC after wait
			tx_buffer[0] = 0x00; 
			HAL_I2C_Master_Transmit(&hi2c1, 0xEC, tx_buffer, 1, 1000);	
			// READ ADC PRESSURE VALUE
			HAL_I2C_Master_Receive(&hi2c1, 0xED, rx_buffer, 3, 1000); // recives 3 bytes of data position 0, 1 and 2 of the buffer
			D1 = ((rx_buffer[0]<<16)|(rx_buffer[1] << 8)|(rx_buffer[2]));
			
			STATE = START_TEMPERATURE_CONV;
			
		break;
		
		case START_TEMPERATURE_CONV:
			
		// INIT TEMPERATURE CONVERSION
			tx_buffer[0] = 0x5A; // Convert D2 (OSR=8192) 18.08ms
			HAL_I2C_Master_Transmit(&hi2c1, 0xEC, tx_buffer, 1, 1000);		
			
			STATE = READ_TEMPERATURE;
					
		break;
		
		
		case READ_TEMPERATURE:
			
			// CMD READ ADC after wait
			tx_buffer[0] = 0x00;
			HAL_I2C_Master_Transmit(&hi2c1, 0xEC, tx_buffer, 1, 1000);	
			// READ ADC TEMPERATURE VALUE
			HAL_I2C_Master_Receive(&hi2c1, 0xED, rx_buffer, 3, 1000); // recives 3 bytes of data position 0, 1 and 2 of the buffer
			D2 = (unsigned int)((rx_buffer[0]<<16)|(rx_buffer[1] << 8)|(rx_buffer[2]));
			
			STATE = CALC_TEMPERATURE;
		
		break;
		
		case CALC_TEMPERATURE:
			
	//Calculate Temp pag 7 datasheet
			dT = (int32_t)(D2 - (((uint32_t)C5) << 8)); // dT  = D2 - T REF
			TEMP = CELSIUS20 + (int32_t)((((int64_t)dT) * ((int64_t)C6)) >> 23); //TEMP = 20°C+dT*TEMPSENS
			OFF = (int64_t)((((int64_t)C2) << 16) + ((((int64_t)C4) >> 7) * ((int64_t)dT))); // 1st order
			SENS = (int64_t)((((int64_t)C1) << 15) + ((((int64_t)C3) >> 8) * ((int64_t)dT))); // 1st order 
			
			STATE = BRANCH_LOW_HIGH_TEMPERATURE;
		
		break;
	
		case BRANCH_LOW_HIGH_TEMPERATURE: 
		
		if (TEMP < CELSIUS20) {
			STATE = LOW_TEMPERATURE;	
			break;
		}
		else {
			STATE = HIGH_TEMPERATURE;
			break;
		}

		case LOW_TEMPERATURE:
				//Ti  = 3 * (((uint64_t)dT) * ((uint64_t)dT)) >> 33; // 32bit*32bit = 64bit
				tmp_TEMPSQR = ((uint64_t)(TEMP - CELSIUS20)) * ((uint64_t)(TEMP - CELSIUS20));
				OFFinterm = 3 * ((tmp_TEMPSQR) >> 1); // 32bit*32bit = 64bit
				SENSinterm =  5 * ((tmp_TEMPSQR) >> 3);
				
		
				
		if(TEMP < -CELSIUS15){
				STATE = VERY_LOW_TEMPERATURE;
				break; 
		}
		else {
			STATE = DUMMY_LOW_TEMPERATURE; //6; //EXIT
			break;
		}
		
		case DUMMY_LOW_TEMPERATURE:
			
			STATE = SECOND_ORDER_COMP;
			
		break;

		case VERY_LOW_TEMPERATURE:
				
				tmp_TEMPSQR = ((int64_t)(TEMP + CELSIUS15))^2;
				OFFinterm += 7 * tmp_TEMPSQR;
				SENSinterm += 4 * tmp_TEMPSQR;
				
				STATE = SECOND_ORDER_COMP; //EXIT
			
		break;

		case HIGH_TEMPERATURE:
		
			//@High Temperature 
				tmp_TEMPSQR = ((uint64_t)(dT)) * ((uint64_t)(dT));
				//Ti = 2 * (tmp_TEMPSQR >> 37);
				tmp_TEMPSQR = ((uint64_t)(TEMP - CELSIUS20)) * ((uint64_t)(TEMP - CELSIUS20));
				OFFinterm = tmp_TEMPSQR >> 4;
				SENSinterm = 0;
				
				STATE = DUMMY_HIGH_TEMPERATURE; // 6; //EXIT

		break;
		
		case DUMMY_HIGH_TEMPERATURE: 
		
			STATE = SECOND_ORDER_COMP;
		
		break;
	
	
		case SECOND_ORDER_COMP:
			
			OFF = OFF - OFFinterm; // 2nd order
			SENS = SENS - SENSinterm; //2nd order
			STATE = OUTPUT_READY;
		
		break;
		
		
		
		
		case OUTPUT_READY:
		//FINAL VALUES 
			//TEMP = (TEMP - Ti) /100;
			PRESSURE = ((((((int64_t)D1) * (SENS)) >> 21) - (OFF)) >> 13)/10;
			new_pressure_value = 1;	
			
			STATE = START_PRESSURE_CONV;
			
		break;
			
		default: 
			break;


		}
	
		if (new_pressure_value == 1)
				return PRESSURE;
		else 
				return -1;

}